package storage_file

import (
	"cake_crm/internal/modules/storage"
	crm "cake_crm/proto"
	"context"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/patrickmn/go-cache"
	"os"
	"strings"
	"time"
)

type Product struct {
	Id              int64                 `json:"id"`
	Article         string                `json:"article"`
	Name            string                `json:"name"`
	Uri             string                `json:"uri"`
	Images          []string              `json:"images"`
	Description     string                `json:"description"`
	Group           int64                 `json:"group"`
	Unit            string                `json:"unit"`
	Inventory       float64               `json:"inventory"`
	Variants        []*crm.Variant        `json:"variants"`
	Characteristics []*crm.Characteristic `json:"characteristics"`
	Category        int64                 `json:"category"`
	Labels          []*crm.Label          `json:"labels"`
}

var (
	ErrProductNotFound = errors.New("product not found")
)

type storageFile struct {
	cache *cache.Cache
}

func NewStorageFile() storage.IStorage {
	return &storageFile{
		cache: cache.New(24*time.Hour, time.Hour),
	}
}

func (s *storageFile) GetCatalog(_ context.Context) ([]*crm.Category, error) {
	data, err := s.readFile("catalog")
	if err != nil {
		return nil, err
	}
	var res []*crm.Category
	if err := json.Unmarshal(data, &res); err != nil {
		return nil, err
	}
	return res, nil
}

func (s *storageFile) GetPositions(_ context.Context, id int64) ([]*crm.Product, error) {
	data, err := s.readFile("products")
	if err != nil {
		return nil, err
	}
	var products []*crm.Product
	if err := json.Unmarshal(data, &products); err != nil {
		return nil, err
	}
	res := make([]*crm.Product, 0, len(products))
	for _, product := range products {
		if id == 0 || product.Category == id {
			s.enrichedProduct(product)
			res = append(res, product)
		}
	}
	return res, nil
}

func (s *storageFile) GetProduct(_ context.Context, id int64) (*crm.Product, error) {
	data, err := s.readFile("products")
	if err != nil {
		return nil, err
	}
	var products []*Product
	if err := json.Unmarshal(data, &products); err != nil {
		return nil, err
	}
	var product *Product
	for _, p := range products {
		if p.Id == id {
			product = p
			break
		}
	}
	if product == nil {
		return nil, ErrProductNotFound
	}
	res := &crm.Product{
		Id:              product.Id,
		Article:         product.Article,
		Name:            product.Name,
		Uri:             product.Uri,
		Images:          product.Images,
		Description:     product.Description,
		GroupedProducts: nil,
		Unit:            product.Unit,
		Inventory:       product.Inventory,
		Variants:        product.Variants,
		Characteristics: product.Characteristics,
		Category:        product.Category,
		Labels:          product.Labels,
	}
	s.enrichedProduct(res)
	for _, p := range products {
		if p.Group == product.Group {
			image := ""
			if len(product.Images) > 0 {
				image = p.Images[0]
			}
			res.GroupedProducts = append(res.GroupedProducts, &crm.GroupedProduct{
				Name:  p.Name,
				Uri:   p.Uri,
				Image: image,
			})
		}
	}
	return res, nil
}

func (s *storageFile) GetBreadcrumbs(ctx context.Context, id int64) ([]*crm.Category, error) {
	product, err := s.GetProduct(ctx, id)
	if err != nil {
		return nil, err
	}
	data, err := s.readFile("catalog")
	if err != nil {
		return nil, err
	}
	var categories []*crm.Category
	if err := json.Unmarshal(data, &categories); err != nil {
		return nil, err
	}
	breadcrumbs := getBreadcrumbs(categories, product.Id)
	breadcrumbs = append(breadcrumbs, &crm.Category{
		Name: product.Name,
	})
	return breadcrumbs, nil
}

func (s *storageFile) enrichedProduct(product *crm.Product) {
	if len(product.Variants) > 1 {
		product.Labels = append(product.Labels, &crm.Label{Name: "Оптом дешевле"})
	}
}

func getBreadcrumbs(categories []*crm.Category, id int64) []*crm.Category {
	for _, category := range categories {
		if category.Id == id {
			category.Children = nil
			return []*crm.Category{category}
		}
		breadcrumbs := getBreadcrumbs(category.Children, id)
		if len(breadcrumbs) != 0 {
			res := make([]*crm.Category, 0, len(breadcrumbs)+1)
			category.Children = nil
			res = append(res, category)
			res = append(res, breadcrumbs...)
			return res
		}
	}
	return nil
}

func (s *storageFile) GetPositionsByText(_ context.Context, text string) ([]*crm.Product, error) {
	data, err := s.readFile("products")
	if err != nil {
		return nil, err
	}
	var products []*crm.Product
	if err := json.Unmarshal(data, &products); err != nil {
		return nil, err
	}
	res := make([]*crm.Product, 0, len(products))
	searchText := strings.TrimSpace(strings.ToLower(text))
	for _, product := range products {
		name := strings.ToLower(product.Name)
		if strings.Contains(name, searchText) {
			s.enrichedProduct(product)
			res = append(res, product)
		}
	}
	return res, nil
}

func (s *storageFile) readFile(name string) ([]byte, error) {
	cacheData, found := s.cache.Get(name)
	if found {
		return cacheData.([]byte), nil
	}
	data, err := os.ReadFile(fmt.Sprintf("resources/%s.json", name))
	if err != nil {
		return nil, err
	}
	s.cache.Set(name, data, cache.DefaultExpiration)
	return data, nil
}
